@@ -1,10 +1,12 @@
 # Oracle Perl Procedure Library
 #
-# Copyright (c) 2001 Jeff Horwitz (jeff@smashing.org).  All rights reserved.
+# Copyright (c) 2001, 2002 Jeff Horwitz (jeff@smashing.org).
+# All rights reserved.
+#
 # This package is free software; you can redistribute it and/or modify it under
 # the same terms as Perl itself.
 
-# $Id: ExtProc.pm,v 1.5 2001/08/20 20:10:12 jhorwitz Exp $
+# $Id: ExtProc.pm,v 1.14 2002/11/20 20:43:06 jhorwitz Exp $
 
 package ExtProc;
 
@@ -24,7 +26,7 @@ our %EXPORT_TAGS = ( 'all' => [ qw(
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 our @EXPORT = qw(
 );
-our $VERSION = '0.91';
+our $VERSION = '0.94';
 
 bootstrap ExtProc $VERSION;
 
@@ -47,7 +49,7 @@ ExtProc - Perl interface to the Oracle Perl External Procedure Library
 
 The ExtProc module provides several functions that return useful data from
 an Oracle database connection.  It is only useful from the Oracle Perl
-External Procedure Library.
+External Procedure Library (extproc_perl).
 
 =head1 FUNCTIONS
 
@@ -71,7 +73,31 @@ Throws a user-defined Oracle exception.  Note that the Perl subroutine will
 probably complete after this function is called, but no return values should
 be accepted by the calling client.
 
-=back 4
+=item context
+
+Returns an OCIExtProcContext object for use with DBI->connect.  When connecting
+to the database that called the external procedure, use the database name
+'extproc', and supply no username or password.  Pass the context returned by
+this function to the DBI->connect method by defining 'ora_context' in the
+attributes parameter.  Use the standard DBI method of connecting when using
+another database.  An example follows:
+
+
+ use DBI;
+ use ExtProc;
+
+ # get the current OCI context
+ my $context = ExtProc::context;
+
+ # connect back to the calling database
+ my $dbh = DBI->connect("dbi:Oracle:extproc", "", "",
+            { 'ora_context' => $context });
+
+NOTE: External procedures are stateless, so there is no concept of a persistent
+connection to the database.  Therefore, you must run the DBI->connect method
+before each query.  This may be automated in the future.
+
+=back
 
 =head1 AUTHOR
 
@@ -79,6 +105,6 @@ Jeff Horwitz <jeff@smashing.org>
 
 =head1 SEE ALSO
 
-perl(1), perlembed(1)
+perl(1), perlembed(1), DBI(3), DBD::Oracle(3)
 
 =cut
