@@ -1,12 +1,14 @@
 /*
  * Oracle Perl Procedure Library
  *
- * Copyright (c) 2001 Jeff Horwitz (jeff@smashing.org).  All rights reserved.
+ * Copyright (c) 2001, 2002 Jeff Horwitz (jeff@smashing.org).
+ * All rights reserved.
+ *
  * This package is free software; you can redistribute it and/or modify it
  * under the same terms as Perl itself.
  */
 
-/* $Id: extproc_perl.c,v 1.7 2001/08/20 19:59:29 jhorwitz Exp $ */
+/* $Id: extproc_perl.c,v 1.17 2002/11/22 19:26:40 jhorwitz Exp $ */
 
 #ifdef __cplusplus
 extern "C" {
@@ -14,6 +16,7 @@ extern "C" {
 #include <stdio.h>
 #include <stdarg.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <time.h>
 #include <oci.h>
 #include <EXTERN.h>
@@ -23,9 +26,10 @@ extern "C" {
 }
 #endif
 
-#define EXTPROC_PERL_VERSION	"0.91"
+#define EXTPROC_PERL_VERSION	"0.94"
 
 static PerlInterpreter *perl;
+static char code_table[256];
 OCIExtProcContext *this_ctx; /* for ExtProc module */
 
 void ora_exception(OCIExtProcContext *ctx, char *msg)
@@ -36,104 +40,35 @@ void ora_exception(OCIExtProcContext *ctx, char *msg)
 	OCIExtProcRaiseExcpWithMsg(ctx, ORACLE_USER_ERR, str, 0);
 }
 
-#if 0
-/* NEEDS REAL ERROR MESSAGES */
-int simple_text_query(OCIExtProcContext *ctx, char *sql, char *res)
-{
-	ocictx oci_ctx;
-	ocictx *oci_ctxp = &oci_ctx;
-	OCIDefine *def1;
-	text out[1024];
-	int err;
-	
-	err = OCIExtProcGetEnv(ctx,
-		&oci_ctxp->envhp,
-		&oci_ctxp->svchp,
-		&oci_ctxp->errhp);
-
-	if (err) {
-		ora_exception(ctx,"getenv");
-		return(err);
-	}
-
-	err = OCIHandleAlloc(oci_ctxp->envhp,
-		(dvoid **)&oci_ctxp->stmtp,
-		OCI_HTYPE_STMT,
-		0,
-		0);
-
-	if (err) {
-		ora_exception(ctx,"handlealloc");
-		return(err);
-	}
-
-	err = OCIStmtPrepare(oci_ctxp->stmtp,
-		oci_ctxp->errhp,
-		(text *) sql,
-		strlen(sql),
-		OCI_NTV_SYNTAX,
-		OCI_DEFAULT);
-
-	if (err) {
-		ora_exception(ctx,"prepare");
-		return(err);
-	}
-
-	err = OCIStmtExecute(oci_ctxp->svchp,
-		oci_ctxp->stmtp,
-		oci_ctxp->errhp,
-		0,
-		0,
-		NULL,
-		NULL,
-		OCI_DEFAULT);
-
-	if (err) {
-		ora_exception(ctx,"exec");
-		return(err);
-	}
-
-	err = OCIDefineByPos(oci_ctxp->stmtp,
-		&def1,
-		oci_ctxp->errhp,
-		1,
-		&out,
-		256,
-		SQLT_STR,
-		(dvoid *) 0,
-		(dvoid *) 0,
-		(dvoid *) 0,
-		OCI_DEFAULT);
-
-	err = OCIStmtFetch(oci_ctxp->stmtp,
-		oci_ctxp->errhp,
-		1,
-		OCI_FETCH_NEXT,
-		OCI_DEFAULT);
-
-	if (err) {
-		ora_exception(ctx,"fetch");
-		return(err);
-	}
-
-	strncpy(res, out, 1024);
-
-	return(0);
-}
-#endif
-
 PerlInterpreter *pl_startup(void)
 {
 	PerlInterpreter *p;
-	char *args[] = { "", BOOTSTRAP_FILE };
+	int argc;
+	char *argv[3];
+	struct stat st;
 
+	/* create interpreter */
 	if((p = perl_alloc()) == NULL) {
 		return(NULL);
 	}
 	PL_perl_destruct_level = 0;
 	perl_construct(p);
-	if (!perl_parse(p, xs_init, 2, args, NULL)) {
-		if (perl_run(p)) {
+
+	/* parse bootstrap file if it exists */
+	if (!stat(BOOTSTRAP_FILE, &st)) {
+		argv[0] = "";
+		argv[1] = BOOTSTRAP_FILE;
+		argc = 2;
+	}
+	else {
+		argv[0] = "";
+		argv[1] = "-e";
+		argv[2] = "0";
+		argc = 3;
+	}
+
+	if (!perl_parse(p, xs_init, argc, argv, NULL)) {
+		if (!perl_run(p)) {
 			return(p);
 		}
 	}
@@ -146,6 +81,17 @@ void pl_shutdown(PerlInterpreter *p)
 	perl_free(p);
 }
 
+static char *get_code(OCIExtProcContext *ctx, char *table, char *buf)
+{
+	OCILobLocator *lobl;
+	char sql[MAX_SIMPLE_QUERY_SQL];
+	int buflen = MAX_SIMPLE_QUERY_RESULT;
+
+	snprintf(sql, MAX_SIMPLE_QUERY_SQL - 18, "select code from %s", table);
+	simple_lob_query(ctx, sql, lobl, buf, &buflen, 0);
+	return(buf);
+}
+
 static char *call_perl_sub(OCIExtProcContext *ctx, char *sub, char **args)
 {
 	STRLEN len;
@@ -190,12 +136,13 @@ static char *call_perl_sub(OCIExtProcContext *ctx, char *sub, char **args)
 	return(retval);
 }
 
+/* entry point from oracle external procedure process */
 char *ora_perl_sub(OCIExtProcContext *ctx, OCIInd *ret_ind, char *sub, ...)
 {
 	int status, n = 0;
 	va_list ap;
 	short ind;
-	char *args[MAXARGS], *retval, user[1024];
+	char *args[MAXARGS], *retval, user[1024], code[MAX_CODE_SIZE];
 
 	/* set OCI context for ExtProc module */
 	this_ctx = ctx;
@@ -215,9 +162,12 @@ char *ora_perl_sub(OCIExtProcContext *ctx, OCIInd *ret_ind, char *sub, ...)
 	va_end(ap);
 
 	/* check for flush request */
-	if (!strncmp(sub, "_flush", 6) && perl) {
-		pl_shutdown(perl);
-		perl = NULL;
+	if (!strncmp(sub, "_flush", 6)) {
+		/* only destroy the interpreter if it exists */
+		if (perl) {
+			pl_shutdown(perl);
+			perl = NULL;
+		}
 		*ret_ind = OCI_IND_NULL;
 		return("\0");
 	}
@@ -228,6 +178,12 @@ char *ora_perl_sub(OCIExtProcContext *ctx, OCIInd *ret_ind, char *sub, ...)
 		return(EXTPROC_PERL_VERSION);
 	}
 
+	/* check for module list request */
+	if (!strncmp(sub, "_modules", 8)) {
+		*ret_ind = OCI_IND_NOTNULL;
+		return(STATIC_MODULES);
+	}
+
 	/* start perl interpreter if necessary */
 	if (!perl) {
 		perl = pl_startup();
@@ -236,13 +192,57 @@ char *ora_perl_sub(OCIExtProcContext *ctx, OCIInd *ret_ind, char *sub, ...)
 			ora_exception(ctx, "pl_startup failed -- check bootstrap file with 'perl -cw'");
 			return("\0");
 		}
+		/* set code table */
+		strncpy(code_table, CODE_TABLE, 255);
 	}
 
-	/* verify that the subroutine is valid (autoloading not supported) */
-	if (!get_sv(sub, FALSE)) {
-		*ret_ind = OCI_IND_NULL;
-		ora_exception(ctx, "invalid subroutine");
-		return("\0");
+	/* the following special subs must be run AFTER the perl interpreter */
+	/* has been initialized */
+
+	/* check for code table change */
+	if (!strncmp(sub, "_codetable", 10)) {
+		if (args[0]) {
+			strncpy(code_table, args[0], 255);
+		}
+		*ret_ind = OCI_IND_NOTNULL;
+		return(code_table);
+	}
+
+	/* check for error message request */
+	if (!strncmp(sub, "_error", 6)) {
+		if (SvTRUE(ERRSV)) {
+			*ret_ind = OCI_IND_NOTNULL;
+			return(SvPV(ERRSV,PL_na));
+		}
+		else {
+			*ret_ind = OCI_IND_NULL;
+			return("\0");
+		}
+	}
+
+	/* check for easter egg */
+	if (!strncmp(sub, "_easteregg", 10)) {
+		*ret_ind = OCI_IND_NOTNULL;
+		return("Sorry, no easter eggs here.  They're a waste of resources. :-D");
+	}
+
+	/*
+	 * verify that the subroutine is valid (autoloading not supported)
+	 * try loading code from database if it isn't initially valid.
+	 */
+	if (!get_cv(sub, FALSE)) {
+		/* load code -- fail silently if no code is available */
+		get_code(ctx, code_table, code);
+
+		/* parse code */
+		eval_pv(code, TRUE);
+
+		/* try again */
+		if (!get_cv(sub, FALSE)) {
+			*ret_ind = OCI_IND_NULL;
+			ora_exception(ctx, "invalid subroutine");
+			return("\0");
+		}
 	}
 
 	/* run subroutine */
